import re
import sys
from setuptools import find_packages, setup

# Using numpy 1.20.0 requires python >= 3.7
if sys.version_info[:2] < (3, 7):
    raise ImportError("""
    This version of coredotfinance no supports python versions less than 3.7.0.
    """)


def find_version(fname):
    """Attempts to find the version number in the file names fname.
    Raises RuntimeError if not found.
    """
    version = ''
    with open(fname, 'r') as fp:
        reg = re.compile(r'__version__ = [\'"]([^\'"]*)[\'"]')
        for line in fp:
            m = reg.match(line)
            if m:
                version = m.group(1)
                break
    if not version:
        raise RuntimeError('Cannot find version information')
    return version


__version__ = find_version('coredotfinance/__init__.py')


def read(fname):
    with open(fname) as fp:
        content = fp.read()
    return content


REQUIREMENTS = [
    'requests>=2; python_version >= "3"',
    'requests>=2; python_version < "3"',
]

PACKAGES = [
    'coredotfinance',
    'coredotfinance.krx',
    'coredotfinance.krx.api',
    'coredotfinance.krx.core',
    'coredotfinance.krx.core.krx_website',
    'coredotfinance.crypto',
    'coredotfinance.crypto.binance',
    'coredotfinance.yahoo'
]

INSTALL_REQUIRES = [
    'pandas >= 0.14',
    'numpy >= 1.8',
    'requests >= 2.7.0',
    'beautifulsoup4 >= 4.9.3',
    'plotly >= 4.14.3',
]

with open('README.md') as f:
    LONG_DESCRIPTION = f.read()

KEYWORDS = [
    'finance',
    'stock',
    'money',
    'krx',
    'bitcoin',
    'cryptocurrency',
    'index'
]

setup(
    name='coredotfinance',
    version=__version__,
    packages=find_packages(),
    url='https://github.com/coredottoday/CoreDotFinance',
    license='',
    author='Core.Today',
    author_email='help@core.today',
    description='Financial data on web',
    keywords=KEYWORDS,
    long_description_content_type = 'text/markdown',
    long_description=LONG_DESCRIPTION,
    install_requires=INSTALL_REQUIRES,
)




import pytest

from coredotfinance.krx.tools import get, per, etf, etn, elw


@pytest.mark.parametrize("stock", [(), ("naver", 20210428, 20210428)])
def test_get(stock):
    assert get(*stock) is not None


@pytest.mark.parametrize("stock", [(), ("naver", 20210428, 20210428)])
def test_per(stock):
    assert per(*stock) is not None


@pytest.mark.parametrize("ETF", [(), ("arirang", 20210428, 20210428)])
def test_etf(ETF):
    assert etf(*ETF) is not None


@pytest.mark.parametrize("ETN", [(), ("qv 미국", 20210428, 20210428)])
def test_etn(ETN):
    assert etn(*ETN) is not None


@pytest.mark.parametrize("ELW", [(), ("KBF937삼성전자콜", 20210428, 20210428)])
def test_elw(ELW):
    assert elw(*ELW) is not None



# -*- coding: utf-8 -*-
from coredotfinance.krx.core.krx_website.info import Info


class Commodity(Info):  # 일반상품: 석유, 금, 배출
    def __init__(self, code, start, end, day, code_to_function, kwargs):
        super(Commodity, self).__init__(start, end, day)
        self.get_requested_data = code_to_function[code]
        self.trade_index = kwargs.get("trade_index", None)
        self.trade_check = kwargs.get("trade_check", None)
        self.inquiry = kwargs.get("inquiry", None)


class Oil(Commodity):  # 석유 [16101~16105]
    def __init__(self, code, start, end, day, **kwargs):
        code_to_function = {
            "16101": self.info_of_entire_items,
            "16102": self.price_trend_of_item,
            "16103": self.trade_performance_per_investor,
            "16104": self.price_domestic_gasstation,
            "16105": self.price_international,
        }
        super().__init__(code, start, end, day, code_to_function, kwargs)
        self.oil = kwargs.get("oil", None)

    def info_of_entire_items(self):
        """전종목 기본정보[16101]"""
        data = {
            "bld": "dbms/MDC/STAT/standard/MDCSTAT14301",
            "secugrpId": self.oil,  # ALL(전체), GA(휘발유), DI(경유), KE(등유)
        }
        return self.update_requested_data(data)

    def price_trend_of_item(self):
        """유종별 시세 추이[16102]"""
        data = {
            "bld": "dbms/MDC/STAT/standard/MDCSTAT14401",
            "secugrpId": self.oil,
            "strtDd": self.start,
            "endDd": self.end,
        }
        return self.update_requested_data(data)

    def trade_performance_per_investor(self):
        """참가자별 거래실적[16103]"""
        if self.inquiry == "일별추이":
            bld = "dbms/MDC/STAT/standard/MDCSTAT14502"
        else:
            bld = "dbms/MDC/STAT/standard/MDCSTAT14501"
        data = {
            "bld": bld,
            "inqTpCd": self.inquiry,  # 1(기간합계), 2(일별추이)
            "trdVolVal": self.trade_index,  # 1(거래량), 2(거래대금)
            "bidAskNet": self.trade_check,  # 1(매도), 2(매수), 3(순매수)
            "secugrpId": self.oil,  # GA(휘발유), DI(경유), KE(등유)
            "strtDd": self.start,
            "endDd": self.end,
        }
        return self.update_requested_data(data)

    def price_domestic_gasstation(self):
        """국내유가(주유소) 동향 [16104]"""
        data = {
            "bld": "dbms/MDC/STAT/standard/MDCSTAT14601",
            "secugrpId": self.oil,  # GA(휘발유), DI(경유), KE(등유)
            "strtDd": self.start,
            "endDd": self.end,
        }
        return self.update_requested_data(data)

    def price_international(self):  # [16105] 국제유가 동향
        pass


class Gold(Commodity):  # 금 [16201~16207]
    def __init__(self, code, start, end, day, **kwargs):
        code_to_function = {
            "16201": self.price_of_entire_items,
            "16202": self.price_trend_of_item,
            "16203": self.info_of_entire_items,
            "16204": self.info_of_item,
            "16205": self.trade_performance_per_investor,
            "16206": self.block_trade,
            "16207": self.price_international,
        }

        super(Gold, self).__init__(code, start, end, day, code_to_function, kwargs)
        self.gold = kwargs.get("gold", None)

    def price_of_entire_items(self):  # [16201] 전종목 시세
        data = {
            "bld": "dbms/MDC/STAT/standard/MDCSTAT14901",
            "trdDd": self.day,
        }
        return self.update_requested_data(data)

    def price_trend_of_item(self):  # [16202] 개별종목 시세 추이
        data = {
            "bld": "dbms/MDC/STAT/standard/MDCSTAT15001",
            "isuCd": self.gold,
            "strtDd": self.start,
            "endDd": self.end,
        }
        return self.update_requested_data(data)

    def info_of_entire_items(self):  # [16203] 전종목 기본정보
        data = {
            "bld": "dbms/MDC/STAT/standard/MDCSTAT15101",
        }
        return self.update_requested_data(data)

    def info_of_item(self):  # [16204] 개별종목 종합정보
        data = {
            "bld": "dbms/MDC/STAT/standard/MDCSTAT15201",
            "isuCd": self.gold,  # KRD040200002(금 99.99K), KRD040201000(미니금 100g)
        }
        """ 데이터 총 3개
        data = {
            'bld': 'dbms/MDC/STAT/standard/MDCSTAT15202',
            'isuCd': self.gold,  # KRD040200002(금 99.99K), KRD040201000(미니금 100g)
        }
        data = {
            'bld': 'dbms/MDC/STAT/standard/MDCSTAT15203',
            'isuCd': self.gold,  # KRD040200002(금 99.99K), KRD040201000(미니금 100g)
        }
        """
        return "Not now"

    def trade_performance_per_investor(self):  # [16205] 투자자별 거래실적
        if self.inquiry == "일별추이":
            bld = "dbms/MDC/STAT/standard/MDCSTAT15302"
        else:
            bld = "dbms/MDC/STAT/standard/MDCSTAT15301"
        data = {
            "bld": bld,
            "strtDd": self.start,
            "endDd": self.end,
            "trdVolVal": self.trade_index,  # 1(거래량), 2(거래대금)
            "bidAskNet": self.trade_check,  # 1(매도), 2(매수), 3(순매수)
        }
        return self.update_requested_data(data)

    def block_trade(self):  # [16206] 협의대량거래실적 추이
        data = {
            "bld": "dbms/MDC/STAT/standard/MDCSTAT15401",
            "strtDd": self.start,
            "endDd": self.end,
        }
        return self.update_requested_data(data)

    def price_international(self):  # [16207] 국제금시세 동향
        if self.inquiry == "현재가":
            data = {
                "bld": "dbms/MDC/STAT/standard/MDCSTAT13901",
                "strtDd": self.start,
                "endDd": self.end,
            }
        else:
            data = {
                "bld": "dbms/MDC/STAT/standard/MDCSTAT13902",
                "strtDd": self.start,
                "endDd": self.end,
            }
        return self.update_requested_data(data)


class CarbonEmission(Commodity):  # 배출권 [16301~16304]
    def __init__(self, code, start, end, day, **kwargs):
        code_to_function = {
            "16301": self.price_of_entire_items,
            "16302": self.price_trend_of_item,
            "16303": self.info_of_entire_items,
            "16304": self.info_of_item,
        }

        super(CarbonEmission, self).__init__(
            code, start, end, day, code_to_function, kwargs
        )
        self.kwargs = kwargs
        self.item = self.kwargs.get(
            "item", None
        )  # isuCd: 개별종목명 [예: KRD050022007(KAU20), KRD050032105(KAU21)]

    def price_of_entire_items(self):  # [16301] 전종목 시세
        data = {
            "bld": "dbms/MDC/STAT/standard/MDCSTAT15601",
            "trdDd": self.day,
            #  'share': 1,  # 1~3(톤 / 천톤 / 백만톤)
            #  'money': 1,  # 1~4(원 / 천원 / 백만원 / 십억원)
        }
        return self.update_requested_data(data)

    def price_trend_of_item(self):  # [16302] 개별종목 시세 추이
        data = {
            "bld": "dbms/MDC/STAT/standard/MDCSTAT15701",
            "isuCd": self.item,  # KRD050022007(KAU20)
            "strtDd": self.start,
            "endDd": self.end,
            #  'share': 1,  # 1~3(톤 / 천톤 / 백만톤)
            #  'money': 1,  # 1~4(원 / 천원 / 백만원 / 십억원)
        }
        return self.update_requested_data(data)

    def info_of_entire_items(self):  # [16303] 전종목 기본정보
        data = {
            "bld": "dbms/MDC/STAT/standard/MDCSTAT15801",
        }
        return self.update_requested_data(data)

    def info_of_item(self):  # [16304] 개별종목 종합정보
        data = {
            "bld": "dbms/MDC/STAT/standard/MDCSTAT15901",
            "isuCd": self.item,  # KRD050022007(KAU20), KRD050032105(KAU21)
        }
        """ 데이터 총 3개
        data = {
            'bld': 'dbms/MDC/STAT/standard/MDCSTAT15902',
            'isuCd': self.isucd, # KRD050022007(KAU20), KRD050032105(KAU21)
        }
        data = {
            'bld': 'dbms/MDC/STAT/standard/MDCSTAT15903',
            'isuCd': self.isucd, # KRD050022007(KAU20), KRD050032105(KAU21)
        }
        """
        return "Not now"



import datetime

import pandas as pd
import plotly.graph_objects as go
from dateutil.relativedelta import relativedelta
from plotly.subplots import make_subplots


def get_date_list(start_day) -> list:
    """시작일부터 오늘까지를 1달 간격으로 나눈 것을 리스트로 만듦"""
    date_list = []
    start = datetime.datetime.strptime(start_day, "%Y%m%d")
    today = datetime.datetime.now()
    first_day = datetime.datetime(today.year, today.month, 1)
    date_list.append(today.strftime("%Y%m%d"))
    date = first_day
    delta = relativedelta(months=1)
    while date > start:
        date_list.append(date.strftime("%Y%m%d"))
        date = date - delta
    date_list.append(start.strftime("%Y%m%d"))
    return date_list


def make_ohlcv_graph(
    df: pd.DataFrame, open="시가", high="고가", low="저가", close="종가", volume="거래량"
) -> None:
    # hovertext 생성
    ohlc_candle_hovertext = []
    volume_bar_hovertext = []
    for i in range(len(df[open])):
        ohlc_candle_hovertext.append(
            f"일자: {df.index[i].date()}<br>시가: {df[open][i]}<br>고가: {df[high][i]}<br>저가: {df[low][i]}<br>종가: {df[close][i]}"
        )
        volume_bar_hovertext.append(f"일자: {df.index[i].date()}<br>거래량: {df[volume][i]}")
    # OHLC 캔들 차트 생성
    ohlc_candle = go.Candlestick(
        x=df.index,
        open=df[open],
        high=df[high],
        low=df[low],
        close=df[close],
        text=ohlc_candle_hovertext,
        hoverinfo="text",
        increasing_line_color="red",
        decreasing_line_color="blue",
    )
    # 거래량 바 차트 생성
    volume_bar = go.Bar(
        x=df.index,
        y=df[volume],
        text=volume_bar_hovertext,
        hoverinfo="text",
    )
    # 그래프 그리기
    fig = make_subplots(rows=2, cols=1, shared_xaxes=True, vertical_spacing=0.02)
    fig.add_trace(ohlc_candle, row=1, col=1)
    fig.add_trace(volume_bar, row=2, col=1)
    fig.update_layout(
        yaxis1_title="가격",
        yaxis2_title="거래량",
        xaxis2_title="기간",
        xaxis1_rangeslider_visible=False,
        xaxis2_rangeslider_visible=True,
        showlegend=False,
        yaxis1=dict(domain=[0.25, 1]),
        yaxis2=dict(domain=[0, 0.2]),
    )
    fig.show()
